/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is LightSMS.
 *
 * The Initial Developer of the Original Code is
 * Byungwook Kang.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/***********************************************************
constants
***********************************************************/

const nsIAutoCompleteSearch = Components.interfaces.nsIAutoCompleteSearch;

const CLASS_ID = Components.ID("{7cb3298e-2538-11dc-8314-0800200c9a67}");

// description
const CLASS_NAME = "Autocomplete from Passwords";

// textual unique identifier
const CONTRACT_ID = "@mozilla.org/autocomplete/search;1?name=password";

/***********************************************************
class definition
***********************************************************/

//class constructor
function Password() {
    this.passwordManager = Components.classes["@mozilla.org/passwordmanager;1"]
                                  .getService(Components.interfaces.nsIPasswordManager);
};

// class definition
Password.prototype = {
  db:{},
  getPassword: function(host,user){
    if(this.db[host]){
      var d=this.db[host][user];
      if(d)return d;
    }
    var e = this.passwordManager.enumerator;
    while (e.hasMoreElements()) {
      try {
        var pass = e.getNext().QueryInterface(Components.interfaces.nsIPassword);
        if (pass.host == host&&pass.user == user) {
           return pass.password;
        }
      }catch(ex) {}
    }
    return null;
  },

  removeValue: function(host,user){
    try{
      this.passwordManager.removeUser(host,user);
    }catch(e){}
    var data=this.db[host];
    delete data[user];
for(a in data){
Components.utils.reportError(a);
}
  },

  startSearch: function(searchString, searchParam, previousResult, listener)
  {
    this.db[searchParam]={};
    var data=this.db[searchParam];
    var candidats = new Array();
    var e = this.passwordManager.enumerator;
    while (e.hasMoreElements()) {
      try {
        var pass = e.getNext().QueryInterface(Components.interfaces.nsIPassword);
        if (pass.host == searchParam&&pass.user.indexOf(searchString) == 0) {
           candidats.push(pass.user);
           data[pass.user]=pass.password;
        }
      }catch(ex) {}
    }
  
    var result = {
      searchParam: searchParam,      
      QueryInterface: function(aIID) {
        if (aIID.equals(nsIAutoCompleteResult) || aIID.equals(nsISupports))
          return this;
        throw Components.results.NS_NOINTERFACE;
      },
      results: null,
      defaultIndex: 0,
      errorDescription: null,
      matchCount: 0,
      searchResult: Components.interfaces.nsIAutoCompleteResult.RESULT_SUCCESS,
      searchString: searchString,
      getCommentAt: function(index) {},
      getStyleAt: function(index) {},
      getValueAt: function(index) {return this.results[index]},
      removeValueAt: function(rowIndex, removeFromDb) {
        this.main.removeValue(this.searchParam,this.results[rowIndex]);
        this.results.splice(rowIndex,1);
      }
    }
    result.main=this;
    result.results = candidats;
    result.matchCount = candidats.length;
    listener.onSearchResult(this, result);
  },
  stopSearch: function() { },

  QueryInterface: function(aIID)
  {
    if (!aIID.equals(nsIAutoCompleteSearch) &&
        !aIID.equals(Components.interfaces.acIPassword) &&
        !aIID.equals(Components.interfaces.nsISupports))
      throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;
  }
};

/***********************************************************
class factory

This object is a member of the global-scope Components.classes.
It is keyed off of the contract ID.
***********************************************************/
var PasswordFactory = {
  createInstance: function (aOuter, aIID)
  {
    if (aOuter != null)
      throw Components.results.NS_ERROR_NO_AGGREGATION;
    return (new Password()).QueryInterface(aIID);
  }
};

/***********************************************************
module definition (xpcom registration)
***********************************************************/
var PasswordModule = {
  registerSelf: function(aCompMgr, aFileSpec, aLocation, aType)
  {
    aCompMgr = aCompMgr.
        QueryInterface(Components.interfaces.nsIComponentRegistrar);
    aCompMgr.registerFactoryLocation(CLASS_ID, CLASS_NAME,
        CONTRACT_ID, aFileSpec, aLocation, aType);
  },

  unregisterSelf: function(aCompMgr, aLocation, aType)
  {
    aCompMgr = aCompMgr.
        QueryInterface(Components.interfaces.nsIComponentRegistrar);
    aCompMgr.unregisterFactoryLocation(CLASS_ID, aLocation);
  },

  getClassObject: function(aCompMgr, aCID, aIID)
  {
    if (!aIID.equals(Components.interfaces.nsIFactory))
      throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

    if (aCID.equals(CLASS_ID))
      return PasswordFactory;

    throw Components.results.NS_ERROR_NO_INTERFACE;
  },
  canUnload: function(aCompMgr) { return true; }
};

/***********************************************************
module initialization

When the application registers the component, this function
is called.
***********************************************************/
function NSGetModule(aCompMgr, aFileSpec) { return PasswordModule; }